<?php

namespace Drupal\migrate_generator_export\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\migrate_generator_export\ExportManager;
use Drush\Commands\DrushCommands as DrushCommandsBase;

/**
 * Defines Drush commands for the migrate_generator_export module.
 */
class DrushCommands extends DrushCommandsBase {

  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Entity export manager service.
   *
   * @var \Drupal\migrate_generator_export\ExportManager
   */
  private $exportManager;

  /**
   * Constructs a new DrushCommands object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\migrate_generator_export\ExportManager $export_manager
   *   Entity export manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ExportManager $export_manager) {
    parent::__construct();
    $this->entityTypeManager = $entity_type_manager;
    $this->exportManager = $export_manager;
  }

  /**
   * Export entity type.
   *
   * @param string $entity_type_id
   *   Entity type to export.
   * @param string $bundle_id
   *   Entity type bundle to export.
   * @param int $entity_id
   *   Entity to export.
   *
   * @command migrate_generator_export:export
   *
   * @usage migrate_generator_export:export node article 1
   *   Node is the type of entity, article is bundle and 1 is ID (optional).
   */
  public function export($entity_type_id, $bundle_id, $entity_id = NULL) {
    if (empty($entity_type_id) && empty($bundle_id)) {
      return;
    }
    $export_config = $this->entityTypeManager->getStorage('entity_type_export_config')->load($entity_type_id . '--' . $bundle_id);
    if (empty($export_config)) {
      $this->messenger()->addError($this->t(
        'In order to export content, please, create corresponding configuration for @type of type @bundle @here.',
          [
            '@type' => $entity_type_id,
            '@bundle' => $bundle_id,
            '@here' => Link::createFromRoute($this->t('here'), 'entity.entity_type_export_config.add_form',
              [
                'entity_type_id' => $entity_type_id,
                'bundle' => $bundle_id,
              ])->toString(),
          ]
        )
      );
      return;
    }
    $batch = [
      'title' => $this->t('Export content'),
      'operations' => [],
      'finished' => 'Drupal\migrate_generator_export\ExportManager::batchFinished',
    ];
    $fields = $export_config->getFields();
    $this->exportManager->getOperations($batch['operations'], $entity_type_id, $bundle_id, $fields, $entity_id, TRUE);
    if (empty($batch['operations'])) {
      return;
    }
    batch_set($batch);
    drush_backend_batch_process();
  }

  /**
   * Export entity type with all related entity types.
   *
   * @param string $entity_type_id
   *   Entity type to export.
   * @param string $bundle_id
   *   Entity type bundle to export.
   *
   * @command migrate_generator_export:export_related
   *
   * @usage migrate_generator_export:export_related node article
   *   Node is the type of entity, article is bundle.
   */
  public function exportRelated($entity_type_id, $bundle_id) {
    if (empty($entity_type_id) && empty($bundle_id)) {
      $this->messenger()->addError($this->t('Please, check entity type ID and entity bundle to be existing.'));
      return;
    }

    $errors = [];
    $export_configs = [];
    $entity_types = $this->entityTypeManager->getDefinitions();
    $this->exportManager->checkDependencies($entity_type_id . '--' . $bundle_id, $entity_types, $export_configs, $errors);
    if (!empty($errors)) {
      foreach ($errors as $error) {
        $this->messenger()->addError($error);
      }
      return;
    }
    if (empty($export_configs)) {
      $this->messenger()->addError($this->t('Please, check entity type ID and entity bundle to be existing.'));
      return;
    }

    $batch = [
      'title' => $this->t('Export content'),
      'operations' => [],
      'finished' => 'Drupal\migrate_generator_export\ExportManager::batchFinished',
    ];

    $storage = $this->entityTypeManager->getStorage('entity_type_export_config');
    foreach ($export_configs as $export_config_id) {
      $export_config = $storage->load($export_config_id);
      if (empty($export_config)) {
        continue;
      }
      $fields = $export_config->getFields();
      list($entity_type_id, $bundle_id) = explode('--', $export_config_id);
      $this->exportManager->getOperations($batch['operations'], $entity_type_id, $bundle_id, $fields, NULL, TRUE);
    }

    if (empty($batch['operations'])) {
      return;
    }

    batch_set($batch);
    drush_backend_batch_process();
  }

}
