<?php

namespace Drupal\migrate_generator_export\Form;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element\Checkboxes;
use Drupal\migrate_generator_export\ExportManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Export form.
 */
class ExportForm extends FormBase implements ContainerInjectionInterface {

  /**
   * Entity Field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $fieldManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $bundleInfoService;

  /**
   * Entity export manager service.
   *
   * @var \Drupal\migrate_generator_export\ExportManager
   */
  private $exportManager;

  /**
   * Constructs a ExportForm object.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $field_manager
   *   Entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundle_info_service
   *   The bundle info service.
   * @param \Drupal\migrate_generator_export\ExportManager $export_manager
   *   Entity export manager service.
   */
  public function __construct(EntityFieldManagerInterface $field_manager, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $bundle_info_service, ExportManager $export_manager) {
    $this->fieldManager = $field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->bundleInfoService = $bundle_info_service;
    $this->exportManager = $export_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('migrate_generator_export.export_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'migrate_generator_export_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#tree'] = TRUE;
    $config_exports = $this->entityTypeManager->getStorage('entity_type_export_config')->loadMultiple();
    if (empty($config_exports)) {
      $form['text'] = [
        '#markup' => $this->t('There is no exports configured. Please, create them on @link page.', ['@link' => Link::createFromRoute($this->t('this'), 'entity.entity_type_export_config.collection')->toString()]),
      ];
      return $form;
    }
    $options = [];
    foreach ($config_exports as $config_export) {
      if (empty($config_export->getFields())) {
        continue;
      }
      $options[$config_export->id()] = $config_export->getName();
    }
    if (empty($options)) {
      $form['text'] = [
        '#markup' => $this->t('There is no exports properly configured. Please, create/update them on @link page.', ['@link' => Link::createFromRoute($this->t('this'), 'entity.entity_type_export_config.collection')->toString()]),
      ];
      return $form;
    }

    $form['exports'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Please, choose exports'),
      '#options' => $options,
    ];

    $form['include_related_content'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Include all referenced content.'),
      '#description' => $this->t('Check this option to export content via entity reference fields.'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Export'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $values = $form_state->getValues();
    if ($values['include_related_content']) {
      $errors = [];
      $dependencies = [];
      $entity_types = $this->entityTypeManager->getDefinitions();
      foreach (Checkboxes::getCheckedCheckboxes($values['exports']) as $export_id) {
        // Check referenced dependencies.
        $this->exportManager->checkDependencies($export_id, $entity_types, $dependencies, $errors);
      }
      if ($errors) {
        foreach ($errors as $referenced_bundle => $error) {
          $form_state->setErrorByName($referenced_bundle, $error);
        }
      }
      if ($dependencies) {
        // Add dependent export configs.
        foreach ($dependencies as $dependency) {
          $values['exports'][$dependency] = $dependency;
        }
        $form_state->setValues($values);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $exports = $form_state->getValue('exports');
    if (empty($exports)) {
      return;
    }
    $batch = [
      'title' => $this->t('Export content'),
      'operations' => [],
      'finished' => 'Drupal\migrate_generator_export\ExportManager::batchFinished',
    ];
    $storage = $this->entityTypeManager->getStorage('entity_type_export_config');
    foreach (Checkboxes::getCheckedCheckboxes($exports) as $export_id) {
      $export = $storage->load($export_id);
      if (!empty($export)) {
        $fields = $export->getFields();
        list($entity_type_id, $bundle_id) = explode('--', $export_id);
        $this->exportManager->getOperations($batch['operations'], $entity_type_id, $bundle_id, $fields);
      }
    }
    if (empty($batch['operations'])) {
      $this->messenger()->addMessage($this->t('No content to export.'));
      return;
    }

    batch_set($batch);
  }

}
